@@ -41,6 +41,7 @@
 //M*/
 
 #include "../precomp.hpp"
+#include "../op_cuda.hpp"
 #include "../op_inf_engine.hpp"
 #include "../ie_ngraph.hpp"
 
@@ -53,87 +54,38 @@
 #include "opencl_kernels_dnn.hpp"
 #endif
 
+#ifdef HAVE_CUDA
+#include "../cuda4dnn/primitives/slice.hpp"
+using namespace cv::dnn::cuda4dnn;
+#endif
+
 namespace cv
 {
 namespace dnn
 {
 
-Range normalizeRange(const Range& input_range, int n)
-{
-    Range range = input_range;
-
-    range.start = std::min(std::max(range.start, -n), n - 1);
-    if (range.start < 0)
-    {
-        range.start += n;
-    }
-
-    range.end = std::min(std::max(range.end, -n), n);
-    if (range.end < 0)
-    {
-        range.end += n;
-    }
-
-    return range;
-}
-
-std::vector<std::vector<cv::Range> > finalizeSliceRange(const MatShape& inpShape, int& axis,
-                                                        const std::vector<std::vector<cv::Range> >& inputSliceRanges)
-{
-    std::vector<std::vector<cv::Range> > sliceRanges = inputSliceRanges;
-    CV_Assert(inpShape.size() > 0);
-    bool axisNeg = (axis < 0);
-    axis = (axis + static_cast<int>(inpShape.size())) % inpShape.size();
-
-    for (size_t i = 0; i < sliceRanges.size(); ++i){
-        std::vector<Range>& ranges = sliceRanges[i];
-        if (axisNeg)
-        {
-            ranges.insert(ranges.begin(), axis, Range::all());
-        }
-
-        for (size_t j = 0; j < ranges.size(); ++j)
-        {
-            int n = inpShape[j];
-            if (n <= 0)
-            {
-                continue;
-            }
-
-            ranges[j] = normalizeRange(ranges[j], n);
-        }
-    }
-
-    return sliceRanges;
-}
-
 class SliceLayerImpl : public SliceLayer
 {
 public:
     SliceLayerImpl(const LayerParams& params)
     {
         setParamsFrom(params);
-        hasSteps = false;
         axis = params.get<int>("axis", 1);
         num_split = params.get<int>("num_split", 0);
-        hasDynamicShapes = params.get<bool>("has_dynamic_shapes", false);
-        shapesInitialized = !hasDynamicShapes;
-
         if (params.has("slice_point"))
         {
             CV_Assert(!params.has("begin") && !params.has("size") && !params.has("end"));
             const DictValue &indicesValue = params.get("slice_point");
-            int size = axis > 0 ? axis + 1 : 1;
             sliceRanges.resize(indicesValue.size() + 1,
-                               std::vector<Range>(size, Range::all()));
+                               std::vector<Range>(axis + 1, Range::all()));
             int prevSlice = 0;
             for (int i = 0; i < indicesValue.size(); ++i)
             {
-                sliceRanges[i][size - 1].start = prevSlice;
-                sliceRanges[i][size - 1].end = indicesValue.get<int>(i);
-                prevSlice = sliceRanges[i][size - 1].end;
+                sliceRanges[i][axis].start = prevSlice;
+                sliceRanges[i][axis].end = indicesValue.get<int>(i);
+                prevSlice = sliceRanges[i][axis].end;
             }
-            sliceRanges.back()[size - 1].start = prevSlice;
+            sliceRanges.back()[axis].start = prevSlice;
         }
         else if (params.has("begin"))
         {
@@ -148,13 +100,14 @@ public:
             {
                 int start = begins.get<int>(i);
                 int sizeOrEnd = sizesOrEnds.get<int>(i);  // It may be negative to reverse indexation.
+                CV_Assert(start >= 0);
 
                 sliceRanges[0][i].start = start;
                 if (params.has("size"))
                 {
                     int size = sizeOrEnd;
                     CV_Assert(size == -1 || size > 0);  // -1 value means range [start, axis_size).
-                    sliceRanges[0][i].end = size > 0 ? (start + size) : INT_MAX;  // We'll finalize a negative value later.
+                    sliceRanges[0][i].end = size > 0 ? (start + size) : -1;  // We'll finalize a negative value later.
                 }
                 else
                 {
@@ -163,32 +116,22 @@ public:
                     sliceRanges[0][i].end = end;  // We'll finalize a negative value later.
                 }
             }
-
-            if (params.has("steps"))
-            {
-                const DictValue &steps = params.get("steps");
-                sliceSteps.resize(1);
-                sliceSteps[0].resize(steps.size());
-
-                for (int i = 0; i < steps.size(); ++i)
-                {
-                    int step = steps.get<int>(i);
-                    CV_Assert(step >= 1);
-                    if (step > 1)
-                        hasSteps = true;
-                    sliceSteps[0][i] = step;
-                }
-            }
         }
     }
 
     virtual bool supportBackend(int backendId) CV_OVERRIDE
     {
-#ifdef HAVE_INF_ENGINE
-        if (backendId == DNN_BACKEND_INFERENCE_ENGINE_NGRAPH)
-            return sliceRanges.size() == 1 && !hasSteps;
+#ifdef HAVE_DNN_IE_NN_BUILDER_2019
+        if (backendId == DNN_BACKEND_INFERENCE_ENGINE_NN_BUILDER_2019)
+            return INF_ENGINE_VER_MAJOR_GE(INF_ENGINE_RELEASE_2019R1) &&
+                sliceRanges.size() == 1 && sliceRanges[0].size() == 4;
 #endif
-        return backendId == DNN_BACKEND_OPENCV;
+#ifdef HAVE_DNN_NGRAPH
+        if (backendId == DNN_BACKEND_INFERENCE_ENGINE_NGRAPH)
+            return sliceRanges.size() == 1;
+#endif
+        return backendId == DNN_BACKEND_OPENCV ||
+               backendId == DNN_BACKEND_CUDA;
     }
 
     bool getMemoryShapes(const std::vector<MatShape> &inputs,
@@ -199,48 +142,31 @@ public:
         CV_Assert(inputs.size() == 1);
         MatShape inpShape = inputs[0];
 
-        int axis_rw = axis;
-        std::vector<std::vector<cv::Range> > sliceRanges_rw = finalizeSliceRange(inpShape, axis_rw, sliceRanges);
-
-        if (!sliceRanges_rw.empty())
+        if (!sliceRanges.empty())
         {
-            outputs.resize(sliceRanges_rw.size(), inpShape);
+            outputs.resize(sliceRanges.size(), inpShape);
             for (int i = 0; i < outputs.size(); ++i)
             {
-                CV_Assert(sliceRanges_rw[i].size() <= inpShape.size());
-                for (int j = 0; j < sliceRanges_rw[i].size(); ++j)
+                CV_Assert(sliceRanges[i].size() <= inpShape.size());
+                for (int j = 0; j < sliceRanges[i].size(); ++j)
                 {
-                    if (shapesInitialized || inpShape[j] > 0)
-                        outputs[i][j] = normalizeRange(sliceRanges_rw[i][j], inpShape[j]).size();
-
-                    if (!sliceSteps.empty() && (i < sliceSteps.size()) && (j < sliceSteps[i].size()) && (sliceSteps[i][j] > 1))
-                        outputs[i][j] = (outputs[i][j] + sliceSteps[i][j] - 1) / sliceSteps[i][j];
+                    outputs[i][j] = clamp(sliceRanges[i][j], inpShape[j]).size();
                 }
             }
         }
         else  // Divide input blob on equal parts by axis.
         {
-            CV_Assert(0 <= axis_rw && axis_rw < inpShape.size());
+            CV_Assert(0 <= axis && axis < inpShape.size());
             int splits = num_split ? num_split : requiredOutputs;
-            CV_Assert(splits > 0 && inpShape[axis_rw] % splits == 0);
-            inpShape[axis_rw] /= splits;
+            CV_Assert(splits > 0 && inpShape[axis] % splits == 0);
+            inpShape[axis] /= splits;
             outputs.resize(splits, inpShape);
         }
         return false;
     }
 
-    bool updateMemoryShapes(const std::vector<MatShape> &inputs) CV_OVERRIDE
-    {
-        shapesInitialized = true;
-        return true;
-    }
-
     void finalize(InputArrayOfArrays inputs_arr, OutputArrayOfArrays outputs_arr) CV_OVERRIDE
     {
-#ifdef HAVE_OPENCL
-        ocl_exec_cache.clear();
-#endif
-
         std::vector<Mat> inputs, outputs;
         inputs_arr.getMatVector(inputs);
         outputs_arr.getMatVector(outputs);
@@ -248,8 +174,7 @@ public:
         CV_Assert(inputs.size() == 1);
         const MatSize& inpShape = inputs[0].size;
 
-        finalSliceRanges = finalizeSliceRange(shape(inputs[0]), axis, sliceRanges);
-
+        finalSliceRanges = sliceRanges;
         if (sliceRanges.empty())
         {
             // Divide input blob on equal parts by axis.
@@ -278,13 +203,10 @@ public:
             // Clamp.
             for (int j = 0; j < finalSliceRanges[i].size(); ++j)
             {
-                finalSliceRanges[i][j] = normalizeRange(finalSliceRanges[i][j], inpShape[j]);
+                finalSliceRanges[i][j] = clamp(finalSliceRanges[i][j], inpShape[j]);
             }
         }
 
-        if (!sliceSteps.empty() && sliceSteps[0].size() != inputs[0].dims)
-            sliceSteps[0].resize(inputs[0].dims, 1);
-
 #if 0
         std::cout << "DEBUG: DNN/Slice: " << outputs.size() << " inpShape=" << inpShape << std::endl;
         for (int i = 0; i < outputs.size(); ++i)
@@ -299,33 +221,26 @@ public:
     }
 
 #ifdef HAVE_OPENCL
-    struct OpenCLExecInfo
+    bool forward_ocl(InputArrayOfArrays inputs_, OutputArrayOfArrays outputs_, OutputArrayOfArrays internals_)
     {
-        std::string kernel_name;
-        std::string build_opts;
-        size_t local_size[2];
-        size_t global_size[2];
+        std::vector<UMat> inputs;
+        std::vector<UMat> outputs;
 
-        OpenCLExecInfo()
-        {
-            local_size[0] = local_size[1] = 0;
-            global_size[0] = global_size[1] = 0;
-        }
-    };
-    std::vector<OpenCLExecInfo> ocl_exec_cache;
-
-    void ocl_prepare(const std::vector<UMat>& inputs, const std::vector<UMat>& outputs)
-    {
-        CV_TRACE_FUNCTION();
+        inputs_.getUMatVector(inputs);
+        outputs_.getUMatVector(outputs);
 
         CV_Assert(outputs.size() == finalSliceRanges.size());
-        ocl_exec_cache.resize(outputs.size());
 
         const UMat& input = inputs[0];
-        const int dims = input.dims;
+        if (input.dims > 5)
+        {
+            CV_LOG_INFO(NULL, "DNN/OpenCL/Slice: implementation doesn't support dims=" << input.dims << ". Fallback to CPU");
+            return false;
+        }
 
         size_t WSZ = 128;
 
+        const int dims = input.dims;
         const int elemSize = (int)input.elemSize();
         String opts0 = cv::format(
                 "-DDIMS=%d -DELEMSIZE=%d",
@@ -335,11 +250,10 @@ public:
         {
             opts0 += cv::format(" -DSRC_STEP_%d=%d", d, (int)input.step[dims - 1 - d]);
         }
+        String kname = cv::format("slice_%d", dims);
         for (size_t i = 0; i < outputs.size(); i++)
         {
-            OpenCLExecInfo& ocl = ocl_exec_cache[i];
-
-            const UMat& output = outputs[i];
+            UMat& output = outputs[i];
             const std::vector<Range>& range = finalSliceRanges[i];
 
             String opts = opts0;
@@ -355,8 +269,6 @@ public:
                 CV_CheckEQ(range[d].size(), (int)output.size[d], "");
             }
 
-            const size_t param_LIMIT_BLOCK_SIZE_PER_WG = WSZ * 64;
-
             int block_dims = 0;
             size_t block_size = elemSize;
             for (int i = dims - 1; i >= 0; --i)
@@ -365,14 +277,12 @@ public:
                     break;
                 block_size *= output.size[i];
                 block_dims++;
-                if (block_size >= param_LIMIT_BLOCK_SIZE_PER_WG)
-                    break;
             }
 
             const size_t total = output.total() * elemSize;
             size_t num_blocks = total / block_size;
 
-            if ((num_blocks <= 8 && block_size >= WSZ * 4) || (block_size >= param_LIMIT_BLOCK_SIZE_PER_WG))
+            if ((num_blocks <= 8 && block_size >= WSZ * 4) || (block_size >= WSZ * 64))
             {
                 // use 1D copy mode
                 opts += cv::format(" -DUSE_COPY_1D=1");
@@ -442,101 +352,23 @@ public:
 
             opts += cv::format(" -DWSZ=%d", (int)WSZ);
 
-            std::ostringstream kernel_suffix;
-            kernel_suffix << dims << 'x' << elemSize << "_bsz" << block_size;
-            kernel_suffix << "__src_";
-            for (int d = 0; d < dims; d++)
-            {
-                kernel_suffix << input.size[dims - 1 - d] << '_';
-            }
-            kernel_suffix << '_';
-            /*for (int d = 0; d < dims; d++)
-            {
-                kernel_suffix << input.step[dims - 1 - d] << '_';
-            }
-            kernel_suffix << '_';*/
+            size_t local[] = { WSZ, 1 };
+            size_t global[] = { WSZ, num_blocks };
 
-            kernel_suffix << "dst_";
-            for (int d = 0; d < dims; d++)
-            {
-                kernel_suffix << output.size[dims - 1 - d] << '_';
-            }
-            /*kernel_suffix << '_';
-            for (int d = 0; d < dims; d++)
-            {
-                kernel_suffix << output.step[dims - 1 - d] << '_';
-            }*/
-            kernel_suffix << "_slice_";
-            for (int d = 0; d < dims; d++)
-            {
-                kernel_suffix << range[dims - 1 - d].start << '_';
-            }
-            for (int d = 0; d < dims; d++)
-            {
-                kernel_suffix << '_' << range[dims - 1 - d].end;
-            }
-
-            std::string kernel_suffix_str = kernel_suffix.str();
-            opts += cv::format(" -DSLICE_KERNEL_SUFFIX=%s", kernel_suffix_str.c_str());
-
-            ocl.kernel_name = cv::format("slice_%s", kernel_suffix_str.c_str());
-            ocl.build_opts = opts;
-            ocl.local_size[0] = WSZ;
-            ocl.local_size[1] = 1;
-            ocl.global_size[0] = WSZ;
-            ocl.global_size[1] = num_blocks;
-        }  // for outputs.size()
-    }  // ocl_prepare
-
-    bool forward_ocl(InputArrayOfArrays inputs_, OutputArrayOfArrays outputs_, OutputArrayOfArrays internals_)
-    {
-        CV_TRACE_FUNCTION();
-
-        if (hasSteps)
-            return false;  // TODO not implemented yet: https://github.com/opencv/opencv/pull/19546
-
-        std::vector<UMat> inputs;
-        std::vector<UMat> outputs;
-
-        inputs_.getUMatVector(inputs);
-        outputs_.getUMatVector(outputs);
-
-        CV_Assert(outputs.size() == finalSliceRanges.size());
-
-        const UMat& input = inputs[0];
-        const int dims = input.dims;
-        if (dims > 5)
-        {
-            CV_LOG_INFO(NULL, "DNN/OpenCL/Slice: implementation doesn't support dims=" << dims << ". Fallback to CPU");
-            return false;
-        }
-
-        if (ocl_exec_cache.empty())
-        {
-            ocl_prepare(inputs, outputs);
-        }
-        CV_CheckEQ(ocl_exec_cache.size(), outputs.size(), "");
-
-        for (size_t i = 0; i < outputs.size(); i++)
-        {
-            const OpenCLExecInfo& ocl = ocl_exec_cache[i];
-
-            UMat& output = outputs[i];
-
-            ocl::Kernel kernel(ocl.kernel_name.c_str(), ocl::dnn::slice_oclsrc, ocl.build_opts);
+            ocl::Kernel kernel(kname.c_str(), ocl::dnn::slice_oclsrc, opts);
             if (kernel.empty())
                 return false;
             bool ret = kernel.args(
                     ocl::KernelArg::PtrReadOnly(input),
                     ocl::KernelArg::PtrWriteOnly(output)
                 )
-                .run_(2, (size_t*)ocl.global_size, (size_t*)ocl.local_size, false);
+                .run(2, global, local, false);
             if (!ret)
                 return false;
         }  // for outputs.size()
 
         return true;
-    }  // forward_ocl
+        }
 #endif
 
     void forward(InputArrayOfArrays inputs_arr, OutputArrayOfArrays outputs_arr, OutputArrayOfArrays internals_arr) CV_OVERRIDE
@@ -553,26 +385,69 @@ public:
 
         const Mat& inpMat = inputs[0];
         CV_Assert(outputs.size() == finalSliceRanges.size());
-
-        if (!hasSteps)
+        for (size_t i = 0; i < outputs.size(); i++)
         {
-            for (size_t i = 0; i < outputs.size(); i++)
-            {
-                inpMat(finalSliceRanges[i]).copyTo(outputs[i]);
-            }
+            inpMat(finalSliceRanges[i]).copyTo(outputs[i]);
+        }
+    }
+
+
+#ifdef HAVE_DNN_IE_NN_BUILDER_2019
+#if INF_ENGINE_VER_MAJOR_GE(INF_ENGINE_RELEASE_2019R1)
+    virtual Ptr<BackendNode> initInfEngine(const std::vector<Ptr<BackendWrapper> >& inputs) CV_OVERRIDE
+    {
+        CV_Assert_N(finalSliceRanges.size() == 1, inputs.size() <= 2);
+
+        std::vector<size_t> axes, offsets, dims;
+        int from, to, step;
+        int numDims = finalSliceRanges[0].size();
+        if (preferableTarget == DNN_TARGET_MYRIAD)
+        {
+            from = axis;
+            to = numDims;
+            step = 1;
         }
         else
         {
-            int dimsNum = inpMat.dims;
-
-            for (size_t i = 0; i < outputs.size(); i++)
-            {
-                std::vector<int> inpIdx(dimsNum, 0);
-                std::vector<int> outIdx(dimsNum, 0);
-                getSliceRecursive(inpMat, inpIdx, finalSliceRanges[i], sliceSteps[i], 0, dimsNum, outputs[i], outIdx);
-            }
+            from = numDims - 1;
+            to = axis - 1;
+            step = -1;
         }
+        for (int i = from; i != to; i += step)
+        {
+            axes.push_back(i);
+            offsets.push_back(finalSliceRanges[0][i].start);
+            dims.push_back(finalSliceRanges[0][i].size());
+        }
+
+        InferenceEngine::Builder::Layer ieLayer(name);
+        ieLayer.setName(name);
+        ieLayer.setType("Crop");
+        ieLayer.getParameters()["axis"] = axes;
+        ieLayer.getParameters()["dim"] = dims;
+        ieLayer.getParameters()["offset"] = offsets;
+        ieLayer.setInputPorts(std::vector<InferenceEngine::Port>(2));
+        ieLayer.setOutputPorts(std::vector<InferenceEngine::Port>(1));
+
+        if (inputs.size() != 2)
+        {
+            std::vector<size_t> outShape(numDims);
+            for (int i = 0; i < numDims; ++i)
+                outShape[i] = finalSliceRanges[0][i].size();
+
+            ieLayer.getInputPorts()[1].setParameter("type", "weights");
+
+            auto shapeSource = InferenceEngine::make_shared_blob<float>({
+                                   InferenceEngine::Precision::FP32, outShape,
+                                   InferenceEngine::Layout::ANY
+                               });
+            shapeSource->allocate();
+            addConstantData("weights", shapeSource, ieLayer);
+        }
+        return Ptr<BackendNode>(new InfEngineBackendNode(ieLayer));
     }
+#endif
+#endif
 
 
 #ifdef HAVE_DNN_NGRAPH
@@ -604,42 +479,33 @@ public:
     }
 #endif  // HAVE_DNN_NGRAPH
 
-private:
-    void getSliceRecursive(const Mat &inpMat, std::vector<int> &inpIdx,
-                           const std::vector<Range> &sliceRanges,
-                           const std::vector<int> &sliceSteps, int dim, int dimsNum,
-                           Mat &outputs, std::vector<int> &outIdx)
+
+#ifdef HAVE_CUDA
+    Ptr<BackendNode> initCUDA(
+        void *context_,
+        const std::vector<Ptr<BackendWrapper>>& inputs,
+        const std::vector<Ptr<BackendWrapper>>& outputs
+    ) override
     {
-        int begin = sliceRanges[dim].start;
-        int end = sliceRanges[dim].end;
-        int step = !sliceSteps.empty() ? sliceSteps[dim] : 1;
+        auto context = reinterpret_cast<csl::CSLContext*>(context_);
 
-        const bool is32F = inpMat.depth() == CV_32F;
-
-        // TODO optimization is required (for 2D tail case at least)
-        for (int k = begin, j = 0; k < end; k += step, j++)
+        std::vector<std::vector<std::size_t>> offsets;
+        for (const auto& ranges : finalSliceRanges)
         {
-            inpIdx[dim] = k;
-            outIdx[dim] = j;
-
-            if (dim + 1 < dimsNum)
-                getSliceRecursive(inpMat, inpIdx, sliceRanges, sliceSteps, dim + 1, dimsNum, outputs, outIdx);
-            else
-            {
-                if (is32F)
-                    outputs.at<float>(outIdx.data()) = inpMat.at<float>(inpIdx.data());
-                else
-                    outputs.at<short>(outIdx.data()) = inpMat.at<short>(inpIdx.data());  // 16F emulation
-            }
+            std::vector<std::size_t> offsets_i;
+            for (const auto& range : ranges)
+                offsets_i.push_back(range.start);
+            offsets.push_back(std::move(offsets_i));
         }
+
+        return make_cuda_node<cuda4dnn::SliceOp>(preferableTarget, std::move(context->stream), std::move(offsets));
     }
+#endif
+
 
 protected:
     // The actual non-negative values determined from @p sliceRanges depends on input size.
     std::vector<std::vector<Range> > finalSliceRanges;
-    bool hasDynamicShapes;
-    bool shapesInitialized;
-    bool hasSteps;
 };
 
 class CropLayerImpl CV_FINAL : public SliceLayerImpl
@@ -666,7 +532,7 @@ public:
         CV_Assert(inputs.size() == 2);
 
         MatShape dstShape = inputs[0];
-        int start = normalize_axis(axis, dstShape);
+        int start = clamp(axis, dstShape);
         for (int i = start; i < dstShape.size(); i++)
         {
             dstShape[i] = inputs[1][i];
@@ -685,7 +551,7 @@ public:
         const Mat &inpSzBlob = inputs[1];
 
         int dims = inpBlob.dims;
-        int start_axis = normalize_axis(axis, dims);
+        int start_axis = clamp(axis, dims);
 
         std::vector<int> offset_final(dims, 0);
         if (offset.size() == 1)
